@@ -57,15 +57,26 @@ static struct {
                     "\\d{2}/\\w{3}/\\d{4}"
                     ")T?"), },
     { "time",    pcrepp(
-          "\\A([\\s\\d]\\d:\\d\\d(?:(?!:\\d)|:\\d\\d(?:\\.\\d{3,6})?Z?))\\b"), },
+          "\\A([\\s\\d]\\d:\\d\\d(?:(?!:\\d)|:\\d\\d(?:[\\.,]\\d{3,6})?Z?))\\b"), },
     /* { "qual", pcrepp("\\A([^\\s:=]+:[^\\s:=,]+(?!,)(?::[^\\s:=,]+)*)"), }, */
-    { "ipv6",    pcrepp("\\A(::|[:\\da-fA-f\\.]+[a-fA-f\\d](?:%\\w+)?)"), },
+    { "ipv6",    pcrepp("\\A(::|[:\\da-fA-F\\.]+[a-fA-F\\d](?:%\\w+)?)"), },
     { "hexd",    pcrepp(
           "\\A([0-9a-fA-F][0-9a-fA-F](?::[0-9a-fA-F][0-9a-fA-F])+)"), },
 
-    { "coln",     pcrepp("\\A(:)"),
+    { "xmlt",   pcrepp(
+            "\\A(<\\??[\\w:]+\\s*(?:[\\w:]+(?:\\s*=\\s*"
+                    "(?:\"((?:\\\\.|[^\"])+)\"|'((?:\\\\.|[^'])+)'|[^>]+)"
+                    "))*\\s*(?:/|\\?)>)"), },
+    { "xmlo",   pcrepp(
+            "\\A(<[\\w:]+\\s*(?:[\\w:]+(?:\\s*=\\s*"
+                    "(?:\"((?:\\\\.|[^\"])+)\"|'((?:\\\\.|[^'])+)'|[^>]+)"
+                    "))*\\s*>)"), },
+
+    { "xmlc",   pcrepp("\\A(</[\\w:]+\\s*>)"), },
+
+    { "coln",    pcrepp("\\A(:)"),
     },
-    { "eq",     pcrepp("\\A(=)"),
+    { "eq",      pcrepp("\\A(=)"),
     },
     { "comm",    pcrepp("\\A(,)"),
     },
@@ -92,22 +103,25 @@ static struct {
     { "rangle",  pcrepp("\\A(\\>)"),
     },
 
-    { "ipv4",    pcrepp("\\A(\\d{1,3}\\.\\d{1,3}\\.\\d{1,3}\\.\\d{1,3})"),
+    { "ipv4", pcrepp("\\A("
+              "(?:(?:25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9]?[0-9])\\.){3}"
+              "(?:25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[1-9]?[0-9])(?![\\d]))"),
     },
+
     { "uuid",    pcrepp(
           "\\A([0-9a-fA-F]{8}(?:-[0-9a-fA-F]{4}){3}-[0-9a-fA-F]{12})"), },
 
     { "vers",    pcrepp(
         "\\A("
             "[0-9]+(?:\\.[0-9]+\\w*){2,}(?:-\\w+)?|"
-            "[0-9]+(?:\\.[0-9]+\\w*)+-\\w+?"
+            "[0-9]+(?:\\.[0-9]+\\w*)+(?<!\\d[eE])-\\w+?"
         ")\\b"),
     },
     { "oct",     pcrepp("\\A(-?0[0-7]+\\b)"),
     },
     { "pcnt",    pcrepp("\\A(-?[0-9]+(\\.[0-9]+)?[ ]*%\\b)"),
     },
-    { "num",     pcrepp("\\A(-?[0-9]+(\\.[0-9]+)?([eE][-+][0-9]+)?)"
+    { "num",     pcrepp("\\A(-?[0-9]+(\\.[0-9]+)?([eE][\\-+][0-9]+)?)"
                         "\\b(?![\\._\\-][a-zA-Z])"), },
     { "hex",     pcrepp("\\A(-?(?:0x|[0-9])[0-9a-fA-F]+)"
                         "\\b(?![\\._\\-][a-zA-Z])"), },
