@@ -107,7 +107,7 @@ public:
  */
 static void copy_to_xclip(void)
 {
-    textview_curses *tc = lnav_data.ld_view_stack.top();
+    textview_curses *tc = lnav_data.ld_view_stack.back();
 
     bookmark_vector<vis_line_t> &bv =
             tc->get_bookmarks()[&textview_curses::BM_USER];
@@ -142,7 +142,7 @@ static void copy_to_xclip(void)
 
 static void back_ten(int ten_minute)
 {
-    textview_curses *   tc  = lnav_data.ld_view_stack.top();
+    textview_curses *   tc  = lnav_data.ld_view_stack.back();
     logfile_sub_source *lss;
 
     lss = dynamic_cast<logfile_sub_source *>(tc->get_sub_source());
@@ -156,21 +156,17 @@ static void back_ten(int ten_minute)
     vis_line_t line = lss->find_from_time(hour);
 
     --line;
-    lnav_data.ld_view_stack.top()->set_top(line);
-}
-
-void update_view_name(void)
-{
-    status_field &sf = lnav_data.ld_top_source.statusview_value_for_field(
-            top_status_source::TSF_VIEW_NAME);
-    textview_curses * tc = lnav_data.ld_view_stack.top();
-
-    sf.set_value("%s ", tc->get_title().c_str());
+    lnav_data.ld_view_stack.back()->set_top(line);
 }
 
 void handle_paging_key(int ch)
 {
-    textview_curses *   tc  = lnav_data.ld_view_stack.top();
+    if (lnav_data.ld_view_stack.empty()) {
+        return;
+    }
+
+    textview_curses *   tc  = lnav_data.ld_view_stack.back();
+    exec_context &ec = lnav_data.ld_exec_context;
     logfile_sub_source *lss = NULL;
     bookmarks<vis_line_t>::type &     bm  = tc->get_bookmarks();
 
@@ -182,51 +178,6 @@ void handle_paging_key(int ch)
 
     /* process the command keystroke */
     switch (ch) {
-        case 'q':
-        case 'Q': {
-            string msg = "";
-
-            if (tc == &lnav_data.ld_views[LNV_DB]) {
-                msg = HELP_MSG_2(v, V, "to switch to the SQL result view");
-            }
-            else if (tc == &lnav_data.ld_views[LNV_HISTOGRAM]) {
-                msg = HELP_MSG_2(i, I, "to switch to the histogram view");
-            }
-            else if (tc == &lnav_data.ld_views[LNV_TEXT]) {
-                msg = HELP_MSG_1(t, "to switch to the text file view");
-            }
-
-            lnav_data.ld_rl_view->set_alt_value(msg);
-            lnav_data.ld_last_view = tc;
-            lnav_data.ld_view_stack.pop();
-            if (lnav_data.ld_view_stack.empty() ||
-                (lnav_data.ld_view_stack.size() == 1 &&
-                 lnav_data.ld_log_source.text_line_count() == 0)) {
-                lnav_data.ld_looping = false;
-            }
-            else {
-                tc = lnav_data.ld_view_stack.top();
-                tc->set_needs_update();
-                if (ch == 'Q') {
-                    text_time_translator *src_view = dynamic_cast<text_time_translator *>(lnav_data.ld_last_view->get_sub_source());
-                    text_time_translator *dst_view = dynamic_cast<text_time_translator *>(tc->get_sub_source());
-                    time_t last_time = 0;
-
-                    if (src_view != NULL && dst_view != NULL) {
-                        last_time = src_view->time_for_row(lnav_data.ld_last_view->get_top());
-                        if (last_time != -1) {
-                            int new_top = dst_view->row_for_time(last_time);
-
-                            tc->set_top(vis_line_t(new_top));
-                        }
-                    }
-                }
-                lnav_data.ld_scroll_broadcaster.invoke(tc);
-                update_view_name();
-            }
-            break;
-        }
-
         case 'a':
             if (lnav_data.ld_last_view == NULL) {
                 alerter::singleton().chime();
@@ -245,7 +196,7 @@ void handle_paging_key(int ch)
             }
             else {
                 textview_curses *tc = lnav_data.ld_last_view;
-                textview_curses *top_tc = lnav_data.ld_view_stack.top();
+                textview_curses *top_tc = lnav_data.ld_view_stack.back();
                 text_time_translator *dst_view = dynamic_cast<text_time_translator *>(tc->get_sub_source());
                 text_time_translator *src_view = dynamic_cast<text_time_translator *>(top_tc->get_sub_source());
 
@@ -293,31 +244,13 @@ void handle_paging_key(int ch)
             lnav_data.ld_rl_view->set_value("Cleared bookmarks");
             break;
 
-        case 'e':
-            moveto_cluster(&bookmark_vector<vis_line_t>::next,
-                           &logfile_sub_source::BM_ERRORS,
-                           tc->get_top());
-            lnav_data.ld_rl_view->set_alt_value(HELP_MSG_2(
-                    w, W,
-                    "to move forward/backward through warning messages"));
-            break;
-
-        case 'E':
-            moveto_cluster(&bookmark_vector<vis_line_t>::prev,
-                           &logfile_sub_source::BM_ERRORS,
-                           tc->get_top());
-            lnav_data.ld_rl_view->set_alt_value(HELP_MSG_2(
-                    w, W,
-                    "to move forward/backward through warning messages"));
-            break;
-
         case 'w':
             moveto_cluster(&bookmark_vector<vis_line_t>::next,
                            &logfile_sub_source::BM_WARNINGS,
                            tc->get_top());
             lnav_data.ld_rl_view->set_alt_value(HELP_MSG_2(
-                    o, O,
-                    "to move forward/backward an hour"));
+                    6, ^,
+                    "to move to next/previous hour boundary"));
             break;
 
         case 'W':
@@ -325,24 +258,8 @@ void handle_paging_key(int ch)
                            &logfile_sub_source::BM_WARNINGS,
                            tc->get_top());
             lnav_data.ld_rl_view->set_alt_value(HELP_MSG_2(
-                    o, O,
-                    "to move forward/backward an hour"));
-            break;
-
-        case 'n':
-            tc->set_top(bm[&textview_curses::BM_SEARCH].next(tc->get_top()));
-            lnav_data.ld_bottom_source.grep_error("");
-            lnav_data.ld_rl_view->set_alt_value(
-                    "Press '" ANSI_BOLD(">") "' or '" ANSI_BOLD("<")
-            "' to scroll horizontally to a search result");
-            break;
-
-        case 'N':
-            tc->set_top(bm[&textview_curses::BM_SEARCH].prev(tc->get_top()));
-            lnav_data.ld_bottom_source.grep_error("");
-            lnav_data.ld_rl_view->set_alt_value(
-                    "Press '" ANSI_BOLD(">") "' or '" ANSI_BOLD("<")
-            "' to scroll horizontally to a search result");
+                    6, ^,
+                    "to move to next/previous hour boundary"));
             break;
 
         case 'y':
@@ -429,7 +346,7 @@ void handle_paging_key(int ch)
                 alerter::singleton().chime();
             }
             else {
-                execute_command("zoom-to " + string(lnav_zoom_strings[lnav_data.ld_zoom_level - 1]));
+                execute_command(ec, "zoom-to " + string(lnav_zoom_strings[lnav_data.ld_zoom_level - 1]));
             }
             break;
 
@@ -438,7 +355,7 @@ void handle_paging_key(int ch)
                 alerter::singleton().chime();
             }
             else {
-                execute_command("zoom-to " + string(lnav_zoom_strings[lnav_data.ld_zoom_level + 1]));
+                execute_command(ec, "zoom-to " + string(lnav_zoom_strings[lnav_data.ld_zoom_level + 1]));
             }
             break;
 
@@ -494,17 +411,6 @@ void handle_paging_key(int ch)
             break;
         }
 
-        case 'm':
-            lnav_data.ld_last_user_mark[tc] = tc->get_top();
-            tc->toggle_user_mark(&textview_curses::BM_USER,
-                                 vis_line_t(lnav_data.ld_last_user_mark[tc]));
-            tc->reload_data();
-
-            lnav_data.ld_rl_view->set_alt_value(HELP_MSG_2(
-                    u, U,
-                    "to move forward/backward through user bookmarks"));
-            break;
-
         case 'J':
             if (lnav_data.ld_last_user_mark.find(tc) ==
                 lnav_data.ld_last_user_mark.end() ||
@@ -588,9 +494,9 @@ void handle_paging_key(int ch)
                                  "----------------\n\n",
                          (int) top, (int) bottom);
                 log_perror(write(STDOUT_FILENO, line_break, strlen(line_break)));
-                for (; top <= bottom; ++top) {
-                    attr_line_t al;
-                    tc->listview_value_for_row(*tc, top, al);
+                vector<attr_line_t> rows(bottom - top);
+                tc->listview_value_for_rows(*tc, top, rows);
+                for (auto &al : rows) {
                     struct line_range lr = find_string_attr_range(
                             al.get_attrs(), &textview_curses::SA_ORIGINAL_LINE);
                     log_perror(write(STDOUT_FILENO, lr.substr(al.get_string()),
@@ -881,6 +787,8 @@ void handle_paging_key(int ch)
                 logfile::iterator ll = lf->begin() + cl;
                 log_data_helper ldh(lss);
 
+                lnav_data.ld_exec_context.ec_top_line = tc->get_top();
+
                 lnav_data.ld_rl_view->clear_possibilities(LNM_COMMAND, "numeric-colname");
                 lnav_data.ld_rl_view->clear_possibilities(LNM_COMMAND, "colname");
 
@@ -923,6 +831,12 @@ void handle_paging_key(int ch)
                      ++iter) {
                     lnav_data.ld_rl_view->add_possibility(LNM_COMMAND, "colname", *iter);
                 }
+                for (auto iter : ldh.ldh_namer->cn_builtin_names) {
+                    if (iter == "col") {
+                        continue;
+                    }
+                    lnav_data.ld_rl_view->add_possibility(LNM_COMMAND, "colname", iter);
+                }
 
                 ldh.clear();
 
@@ -963,6 +877,7 @@ void handle_paging_key(int ch)
             add_filter_possibilities(tc);
             add_mark_possibilities();
             add_config_possibilities();
+            add_env_possibilities(LNM_COMMAND);
             lnav_data.ld_mode = LNM_COMMAND;
             lnav_data.ld_rl_view->focus(LNM_COMMAND, ":");
             break;
@@ -983,6 +898,7 @@ void handle_paging_key(int ch)
                 tc == &lnav_data.ld_views[LNV_DB] ||
                 tc == &lnav_data.ld_views[LNV_SCHEMA]) {
                 textview_curses &log_view = lnav_data.ld_views[LNV_LOG];
+                lnav_data.ld_exec_context.ec_top_line = tc->get_top();
 
                 lnav_data.ld_mode = LNM_SQL;
                 setup_logline_table();
@@ -1009,12 +925,11 @@ void handle_paging_key(int ch)
 
             lnav_data.ld_mode = LNM_EXEC;
 
+            lnav_data.ld_exec_context.ec_top_line = tc->get_top();
             lnav_data.ld_rl_view->clear_possibilities(LNM_EXEC, "__command");
             find_format_scripts(lnav_data.ld_config_paths, scripts);
-            for (map<string, vector<script_metadata> >::iterator iter = scripts.begin();
-                 iter != scripts.end();
-                 ++iter) {
-                lnav_data.ld_rl_view->add_possibility(LNM_EXEC, "__command", iter->first);
+            for (const auto &iter : scripts) {
+                lnav_data.ld_rl_view->add_possibility(LNM_EXEC, "__command", iter.first);
             }
             add_view_text_possibilities(LNM_EXEC, "*", tc);
             add_env_possibilities(LNM_EXEC);
@@ -1068,7 +983,6 @@ void handle_paging_key(int ch)
 
         case 'i':
             if (toggle_view(&lnav_data.ld_views[LNV_HISTOGRAM])) {
-                lss->text_filters_changed();
                 lnav_data.ld_rl_view->set_alt_value(
                         HELP_MSG_2(z, Z, "to zoom in/out"));
             }
@@ -1080,30 +994,27 @@ void handle_paging_key(int ch)
         case 'I':
         {
             time_t log_top = lnav_data.ld_top_time;
+            hist_source2 &hs = lnav_data.ld_hist_source2;
 
             if (toggle_view(&lnav_data.ld_views[LNV_HISTOGRAM])) {
-                hist_source2 &hs = lnav_data.ld_hist_source2;
-
-                lss->text_filters_changed();
-                tc = lnav_data.ld_view_stack.top();
+                tc = lnav_data.ld_view_stack.back();
                 tc->set_top(vis_line_t(hs.row_for_time(log_top)));
             }
             else {
                 textview_curses &hist_tc = lnav_data.ld_views[LNV_HISTOGRAM];
                 textview_curses &log_tc = lnav_data.ld_views[LNV_LOG];
-                time_t hist_top =
-                        lnav_data.ld_hist_source2.time_for_row(hist_tc.get_top());
                 lss = &lnav_data.ld_log_source;
-                log_tc.set_top(lss->find_from_time(hist_top));
-                log_tc.set_needs_update();
+                time_t hist_top_time = hs.time_for_row(hist_tc.get_top());
+                time_t curr_top_time = lss->time_for_row(log_tc.get_top());
+                if (hs.row_for_time(hist_top_time) != hs.row_for_time(curr_top_time)) {
+                    vis_line_t new_top = lss->find_from_time(hist_top_time);
+                    log_tc.set_top(new_top);
+                    log_tc.set_needs_update();
+                }
             }
         }
             break;
 
-        case KEY_CTRL_G:
-            toggle_view(&lnav_data.ld_views[LNV_GRAPH]);
-            break;
-
         case '?':
             toggle_view(&lnav_data.ld_views[LNV_HELP]);
             break;
@@ -1205,8 +1116,17 @@ void handle_paging_key(int ch)
             }
             break;
 
+        case 'x':
+            if (tc->toggle_hide_fields()) {
+                lnav_data.ld_rl_view->set_value("Showing hidden fields");
+            } else {
+                lnav_data.ld_rl_view->set_value("Hiding hidden fields");
+            }
+            tc->set_needs_update();
+            break;
+
         case 'X':
-            lnav_data.ld_rl_view->set_value(execute_command("close"));
+            lnav_data.ld_rl_view->set_value(execute_command(ec, "close"));
             break;
 
         case '\\':
@@ -1271,10 +1191,14 @@ void handle_paging_key(int ch)
             break;
 
         case KEY_CTRL_W:
-            execute_command(lnav_data.ld_views[LNV_LOG].get_word_wrap() ?
+            execute_command(ec, lnav_data.ld_views[LNV_LOG].get_word_wrap() ?
                             "disable-word-wrap" : "enable-word-wrap");
             break;
 
+        case KEY_CTRL_P:
+            lnav_data.ld_preview_hidden = !lnav_data.ld_preview_hidden;
+            break;
+
         default:
             log_warning("unhandled %d", ch);
             lnav_data.ld_rl_view->set_value("Unrecognized keystroke, press "
