@@ -46,6 +46,7 @@ listview_curses::listview_curses()
     : lv_source(NULL),
       lv_overlay_source(NULL),
       lv_window(NULL),
+      lv_x(0),
       lv_y(0),
       lv_top(0),
       lv_left(0),
@@ -182,7 +183,7 @@ bool listview_curses::handle_key(int ch)
 
 void listview_curses::do_update(void)
 {
-    if (this->lv_window == NULL) {
+    if (this->lv_window == NULL || this->lv_height == 0) {
         return;
     }
 
@@ -206,6 +207,8 @@ void listview_curses::do_update(void)
         row_count = this->get_inner_height();
         row   = this->lv_top;
         bottom = y + height;
+        vector<attr_line_t> rows(min((size_t) height, row_count - (int) this->lv_top));
+        this->lv_source->listview_value_for_rows(*this, row, rows);
         while (y < bottom) {
             lr.lr_start = this->lv_left;
             lr.lr_end   = this->lv_left + wrap_width;
@@ -214,16 +217,15 @@ void listview_curses::do_update(void)
                     *this,
                     y - vis_line_t(this->lv_y),
                     overlay_line)) {
-                this->mvwattrline(this->lv_window, y, 0, overlay_line, lr);
+                this->mvwattrline(this->lv_window, y, this->lv_x, overlay_line, lr);
                 overlay_line.clear();
                 ++y;
             }
             else if (row < (int)row_count) {
-                attr_line_t al;
+                attr_line_t &al = rows[row - this->lv_top];
 
-                this->lv_source->listview_value_for_row(*this, row, al);
                 do {
-                    this->mvwattrline(this->lv_window, y, 0, al, lr);
+                    this->mvwattrline(this->lv_window, y, this->lv_x, al, lr);
                     if (this->lv_word_wrap) {
                         wmove(this->lv_window, y, wrap_width);
                         wclrtoeol(this->lv_window);
@@ -235,7 +237,7 @@ void listview_curses::do_update(void)
                 ++row;
             }
             else {
-                wmove(this->lv_window, y, 0);
+                wmove(this->lv_window, y, this->lv_x);
                 wclrtoeol(this->lv_window);
                 ++y;
             }
@@ -282,12 +284,12 @@ void listview_curses::do_update(void)
                 mvwaddch(this->lv_window, gutter_y, width - 1, ch);
                 wattroff(this->lv_window, attrs);
             }
-            wmove(this->lv_window, this->lv_y + height - 1, 0);
+            wmove(this->lv_window, this->lv_y + height - 1, this->lv_x);
         }
 
         if (this->lv_show_bottom_border) {
             mvwchgat(this->lv_window,
-                this->lv_y + height - 1, 0, width - 1, A_UNDERLINE, 0, NULL);
+                this->lv_y + height - 1, this->lv_x, width - 1, A_UNDERLINE, 0, NULL);
         }
 
         this->lv_needs_update = false;
@@ -311,7 +313,7 @@ void listview_curses::do_update(void)
                     *this,
                     y - vis_line_t(this->lv_y),
                 overlay_line)) {
-                this->mvwattrline(this->lv_window, y, 0, overlay_line, lr);
+                this->mvwattrline(this->lv_window, y, this->lv_x, overlay_line, lr);
                 overlay_line.clear();
             }
             ++y;
