@@ -182,8 +182,12 @@ void log_host_info(void)
     log_info("  gid=%d", getgid());
     log_info("  euid=%d", geteuid());
     log_info("  egid=%d", getegid());
-    getcwd(cwd, sizeof(cwd));
-    log_info("  cwd=%s", cwd);
+    if (getcwd(cwd, sizeof(cwd)) == NULL) {
+        log_info("  ERROR: getcwd failed");
+    }
+    else {
+        log_info("  cwd=%s", cwd);
+    }
     log_info("Executable:");
     log_info("  version=%s", VCS_PACKAGE_STRING);
 
@@ -304,6 +308,7 @@ static void sigabrt(int sig)
     time_t curr_time;
 
     if (lnav_log_crash_dir == NULL) {
+        printf("%*s", (int) log_ring.lr_length, log_ring.lr_data);
         return;
     }
 
