@@ -45,13 +45,13 @@ static const char *type_to_string(int type)
 {
     switch (type) {
     case SQLITE_FLOAT:
-        return "float";
+        return "FLOAT";
 
     case SQLITE_INTEGER:
-        return "integer";
+        return "INTEGER";
 
     case SQLITE_TEXT:
-        return "text";
+        return "TEXT";
     }
 
     ensure("Invalid sqlite type");
@@ -64,36 +64,52 @@ std::string log_vtab_impl::get_table_statement(void)
     std::vector<log_vtab_impl::vtab_column> cols;
     std::vector<log_vtab_impl::vtab_column>::const_iterator iter;
     std::ostringstream oss;
+    size_t max_name_len = 15;
 
     oss << "CREATE TABLE " << this->get_name().to_string() << " (\n"
-        << "  log_line integer PRIMARY KEY,\n"
-        << "  log_part text collate naturalnocase,\n"
-        << "  log_time datetime,\n"
-        << "  log_actual_time datetime hidden,\n"
-        << "  log_idle_msecs int,\n"
-        << "  log_level text collate loglevel,\n"
-        << "  log_mark boolean,\n";
+        << "  log_line        INTEGER  PRIMARY KEY,\n"
+        << "  log_part        TEXT     COLLATE naturalnocase,\n"
+        << "  log_time        DATETIME,\n"
+        << "  log_actual_time DATETIME HIDDEN,\n"
+        << "  log_idle_msecs  INTEGER,\n"
+        << "  log_level       TEXT     COLLATE loglevel,\n"
+        << "  log_mark        BOOLEAN,\n"
+        << "  -- BEGIN Format-specific fields:\n";
     this->get_columns(cols);
     this->vi_column_count = cols.size();
+    for (iter = cols.begin(); iter != cols.end(); iter++) {
+        max_name_len = std::max(max_name_len, iter->vc_name.length());
+    }
     for (iter = cols.begin(); iter != cols.end(); iter++) {
         auto_mem<char, sqlite3_free> coldecl;
         auto_mem<char, sqlite3_free> colname;
+        string comment;
+
+        if (!iter->vc_comment.empty()) {
+            comment.append(" -- ")
+                   .append(iter->vc_comment);
+        }
 
         colname = sql_quote_ident(iter->vc_name.c_str());
-        coldecl = sqlite3_mprintf("  %s %s %s collate %Q,\n",
+        coldecl = sqlite3_mprintf("  %-*s %-7s %s COLLATE %-15Q,%s\n",
+                                  max_name_len,
                                   colname.in(),
                                   type_to_string(iter->vc_type),
                                   iter->vc_hidden ? "hidden" : "",
                                   (iter->vc_collator == NULL ||
                                    iter->vc_collator[0] == '\0') ?
-                                  "BINARY" : iter->vc_collator);
+                                  "BINARY" : iter->vc_collator,
+                                  comment.c_str());
         oss << coldecl;
     }
-    oss << "  log_path text hidden collate naturalnocase,\n"
-        << "  log_text text hidden,\n"
-        << "  log_body text hidden\n"
+    oss << "  -- END Format-specific fields\n"
+        << "  log_path        TEXT HIDDEN COLLATE naturalnocase,\n"
+        << "  log_text        TEXT HIDDEN,\n"
+        << "  log_body        TEXT HIDDEN\n"
         << ");\n";
 
+    log_debug("log_vtab_impl.get_table_statement() -> %s", oss.str().c_str());
+
     return oss.str();
 }
 
@@ -226,7 +242,8 @@ static int vt_next(sqlite3_vtab_cursor *cur)
     do {
         log_cursor_latest = vc->log_cursor;
         if (((log_cursor_latest.lc_curr_line % 1024) == 0) &&
-            log_vtab_progress_callback(log_cursor_latest)) {
+            (log_vtab_progress_callback != NULL &&
+             log_vtab_progress_callback(log_cursor_latest))) {
             break;
         }
         done = vt->vi->next(vc->log_cursor, *vt->lss);
@@ -649,7 +666,7 @@ static int vt_best_index(sqlite3_vtab *tab, sqlite3_index_info *p_info)
         p_info->idxNum = argvInUse;
         p_info->idxStr = (char *) index_copy;
         p_info->needToFreeIdxStr = 1;
-        p_info->estimatedCost = 1.0;
+        p_info->estimatedCost = 10.0;
     }
 
     return SQLITE_OK;
