@@ -61,7 +61,8 @@ throw (error)
       lf_is_closed(false),
       lf_logline_observer(NULL),
       lf_logfile_observer(NULL),
-      lf_longest_line(0)
+      lf_longest_line(0),
+      lf_text_format(TF_UNKNOWN)
 {
     require(filename.size() > 0);
 
@@ -184,8 +185,7 @@ void logfile::process_prefix(off_t offset, shared_buffer_ref &sbr)
                     this->lf_index.size(),
                     (*iter)->get_name().get());
 
-                this->lf_format =
-                    auto_ptr<log_format>((*iter)->specialized());
+                this->lf_format = (*iter)->specialized();
                 this->set_format_base_time(this->lf_format.get());
                 this->lf_content_id = hash_string(string(sbr.get_data(), sbr.length()));
 
@@ -334,6 +334,14 @@ throw (line_buffer::error, logfile::error)
         while (this->lf_line_buffer.read_line(off, sbr, &lv)) {
             size_t old_size = this->lf_index.size();
 
+            if (old_size == 0) {
+                shared_buffer_ref avail_sbr;
+
+                this->lf_line_buffer.read_available(avail_sbr);
+                this->lf_text_format = detect_text_format(
+                    avail_sbr.get_data(), avail_sbr.length());
+            }
+
             this->lf_longest_line = std::max(this->lf_longest_line, sbr.length());
             this->lf_partial_line = lv.lv_partial;
             this->process_prefix(last_off, sbr);
