@@ -61,6 +61,7 @@
 #include "lnav_util.hh"
 #include "ansi_scrubber.hh"
 #include "readline_curses.hh"
+#include "spookyhash/SpookyV2.h"
 
 using namespace std;
 
@@ -230,8 +231,8 @@ char *readline_context::completion_generator(const char *text, int state)
                            strncmp : strncasecmp);
                 // Check for an exact match and for the quoted version.
                 if (cmpfunc(text, poss_str, len) == 0 ||
-                        ((strchr(loaded_context->rc_quote_chars, poss_str[0]) != NULL) &&
-                         cmpfunc(text, &poss_str[1], len) == 0)) {
+                    ((strchr(loaded_context->rc_quote_chars, poss_str[0]) != NULL) &&
+                     cmpfunc(text, &poss_str[1], len) == 0)) {
                     matches.push_back(*iter);
                 }
             }
@@ -270,30 +271,35 @@ char **readline_context::attempted_completion(const char *text,
         arg_possibilities = &loaded_context->rc_possibilities["__command"];
         rl_completion_append_character = loaded_context->rc_append_character;
     }
-    else if (loaded_context->rc_possibilities.find("*") !=
-        loaded_context->rc_possibilities.end()) {
-        arg_possibilities = &loaded_context->rc_possibilities["*"];
-        rl_completion_append_character = loaded_context->rc_append_character;
-    }
     else {
         char * space;
         string cmd;
 
         rl_completion_append_character = 0;
         space = strchr(rl_line_buffer, ' ');
-        require(space != NULL);
+        if (space == nullptr) {
+            space = rl_line_buffer + strlen(rl_line_buffer);
+        }
         cmd = string(rl_line_buffer, space - rl_line_buffer);
 
-        vector<string> &proto = loaded_context->rc_prototypes[cmd];
+        auto iter = loaded_context->rc_prototypes.find(cmd);
 
-        if (proto.empty()) {
-            arg_possibilities = NULL;
-        }
-        else if (proto[0] == "filename") {
-            return NULL; /* XXX */
-        }
-        else {
-            arg_possibilities = &(loaded_context->rc_possibilities[proto[0]]);
+        if (iter == loaded_context->rc_prototypes.end()) {
+            if (loaded_context->rc_possibilities.find("*") !=
+                loaded_context->rc_possibilities.end()) {
+                arg_possibilities = &loaded_context->rc_possibilities["*"];
+                rl_completion_append_character = loaded_context->rc_append_character;
+            }
+        } else {
+            vector<string> &proto = loaded_context->rc_prototypes[cmd];
+
+            if (proto.empty()) {
+                arg_possibilities = NULL;
+            } else if (proto[0] == "filename") {
+                return NULL; /* XXX */
+            } else {
+                arg_possibilities = &(loaded_context->rc_possibilities[proto[0]]);
+            }
         }
     }
 
@@ -458,6 +464,8 @@ void readline_curses::start(void)
         }
         else {
             if (FD_ISSET(STDIN_FILENO, &ready_rfds)) {
+                static uint64_t last_h1, last_h2;
+
                 struct itimerval itv;
 
                 if (current_context == this->rc_contexts.end()) {
@@ -478,13 +486,21 @@ void readline_curses::start(void)
                     rl_callback_handler_remove();
                 }
                 else {
-                    if (sendcmd(this->rc_command_pipe[RCF_SLAVE],
-                                'l',
-                                rl_line_buffer,
-                                rl_end) != 0) {
+                    uint64_t h1 = 1, h2 = 2;
+
+                    SpookyHash::Hash128(rl_line_buffer, rl_end, &h1, &h2);
+
+                    if (h1 == last_h1 && h2 == last_h2) {
+                        // do nothing
+                    } else if (sendcmd(this->rc_command_pipe[RCF_SLAVE],
+                                       'l',
+                                       rl_line_buffer,
+                                       rl_end) != 0) {
                         perror("line: write failed");
                         _exit(1);
                     }
+                    last_h1 = h1;
+                    last_h2 = h2;
                 }
             }
             if (FD_ISSET(this->rc_command_pipe[RCF_SLAVE], &ready_rfds)) {
@@ -682,7 +698,12 @@ void readline_curses::check_poll_set(const vector<struct pollfd> &pollfds)
 
         rc = read(this->rc_pty[RCF_MASTER], buffer, sizeof(buffer));
         if (rc > 0) {
+            int old_x = this->vc_x;
+
             this->map_output(buffer, rc);
+            if (this->vc_x != old_x) {
+                this->rc_change.invoke(this);
+            }
         }
     }
     if (pollfd_ready(pollfds, this->rc_command_pipe[RCF_MASTER])) {
@@ -731,9 +752,7 @@ void readline_curses::check_poll_set(const vector<struct pollfd> &pollfds)
                     break;
 
                 case 't':
-                    if (this->rc_value != old_value) {
-                        this->rc_timeout.invoke(this);
-                    }
+                    this->rc_timeout.invoke(this);
                     break;
 
                 case 'd':
@@ -749,6 +768,8 @@ void readline_curses::check_poll_set(const vector<struct pollfd> &pollfds)
                 case 'l':
                     this->rc_line_buffer = &msg[2];
                     this->rc_change.invoke(this);
+                    this->rc_matches.clear();
+                    this->rc_display_match.invoke(this);
                     break;
 
                 case 'n':
