@@ -713,7 +713,7 @@ static int read_last_search(yajlpp_parse_context *ypc, const unsigned char *str,
 
     if (view_index < LNV__MAX && !regex.empty()) {
         execute_search((lnav_view_t)view_index, regex);
-        lnav_data.ld_views[view_index].set_follow_search(false);
+        lnav_data.ld_views[view_index].set_follow_search_for(-1);
     }
 
     return 1;
@@ -771,9 +771,9 @@ static int read_commands(yajlpp_parse_context *ypc, const unsigned char *str, si
             ypc->get_path_fragment(-3));
     view_index = view_name - lnav_view_strings;
     bool active = ensure_view(&lnav_data.ld_views[view_index]);
-    execute_command(cmdline);
+    execute_command(lnav_data.ld_exec_context, cmdline);
     if (!active) {
-        lnav_data.ld_view_stack.pop();
+        lnav_data.ld_view_stack.pop_back();
     }
 
     return 1;
