@@ -103,7 +103,7 @@ const char *sql_keywords[] = {
     "FROM",
     "FULL",
     "GLOB",
-    "GROUP",
+    "GROUP BY",
     "HAVING",
     "IF",
     "IGNORE",
@@ -134,7 +134,7 @@ const char *sql_keywords[] = {
     "OFFSET",
     "ON",
     "OR",
-    "ORDER",
+    "ORDER BY",
     "OUTER",
     "PLAN",
     "PRAGMA",
@@ -171,6 +171,7 @@ const char *sql_keywords[] = {
     "VIRTUAL",
     "WHEN",
     "WHERE",
+    "WITH",
 
     NULL
 };
@@ -201,6 +202,7 @@ const char *sql_function_names[] = {
     "lower(",
     "ltrim(",
     "nullif(",
+    "printf(",
     "quote(",
     "random()",
     "randomblob(",
@@ -510,7 +512,7 @@ ssize_t sql_strftime(char *buffer, size_t buffer_size, time_t time, int millis,
     buffer[index++] = '0' + ((millis / 100) % 10);
     buffer[index++] = '0' + ((millis /  10) % 10);
     buffer[index++] = '0' + ((millis /   1) % 10);
-    buffer[index++] = '\0';
+    buffer[index] = '\0';
 
     return index;
 }
@@ -548,6 +550,17 @@ void sql_install_logger(void)
 #endif
 }
 
+bool sql_ident_needs_quote(const char *ident)
+{
+    for (int lpc = 0; ident[lpc]; lpc++) {
+        if (!isalnum(ident[lpc]) && ident[lpc] != '_') {
+            return true;
+        }
+    }
+
+    return false;
+}
+
 char *sql_quote_ident(const char *ident)
 {
     bool needs_quote = false;
@@ -597,11 +610,11 @@ char *sql_quote_ident(const char *ident)
     return retval;
 }
 
-void sql_execute_script(sqlite3 *db,
+void sql_compile_script(sqlite3 *db,
                         const char *src_name,
                         const char *script_orig,
-                        std::vector<std::string> &errors)
-{
+                        std::vector<sqlite3_stmt *> &stmts,
+                        std::vector<std::string> &errors) {
     const char *script = script_orig;
 
     while (script != NULL && script[0]) {
@@ -629,28 +642,81 @@ void sql_execute_script(sqlite3 *db,
             const char *errmsg = sqlite3_errmsg(db);
             auto_mem<char> full_msg;
 
-            if (asprintf(full_msg.out(), "error:%s:%d:%s", src_name, line_number, errmsg) == -1) {
+            if (asprintf(full_msg.out(), "error:%s:%d:%s", src_name,
+                         line_number, errmsg) == -1) {
                 log_error("unable to allocate error message");
                 break;
             }
             errors.push_back(full_msg.in());
             break;
-        }
-        else if (script == tail) {
+        } else if (script == tail) {
             break;
-        }
-        else if (stmt == NULL) {
+        } else if (stmt == NULL) {
 
+        } else {
+            stmts.push_back(stmt.release());
         }
-        else {
-            retcode = sqlite3_step(stmt.in());
+
+        script = tail;
+    }
+}
+
+void sql_execute_script(sqlite3 *db,
+                        const std::vector<sqlite3_stmt *> &stmts,
+                        std::vector<std::string> &errors)
+{
+    map<string, string> lvars;
+
+    for (sqlite3_stmt *stmt : stmts) {
+        bool done = false;
+        int param_count;
+
+        sqlite3_clear_bindings(stmt);
+
+        param_count = sqlite3_bind_parameter_count(stmt);
+        for (int lpc = 0; lpc < param_count; lpc++) {
+            const char *name;
+
+            name = sqlite3_bind_parameter_name(stmt, lpc + 1);
+            if (name[0] == '$') {
+                map<string, string>::iterator iter;
+                const char *env_value;
+
+                if ((iter = lvars.find(&name[1])) != lvars.end()) {
+                    sqlite3_bind_text(stmt, lpc + 1,
+                                      iter->second.c_str(), -1,
+                                      SQLITE_TRANSIENT);
+                } else if ((env_value = getenv(&name[1])) != NULL) {
+                    sqlite3_bind_text(stmt, lpc + 1,
+                                      env_value, -1,
+                                      SQLITE_TRANSIENT);
+                } else {
+                    sqlite3_bind_null(stmt, lpc + 1);
+                }
+            } else {
+                sqlite3_bind_null(stmt, lpc + 1);
+            }
+        }
+        while (!done) {
+            int retcode = sqlite3_step(stmt);
             switch (retcode) {
                 case SQLITE_OK:
                 case SQLITE_DONE:
+                    done = true;
                     break;
 
-                case SQLITE_ROW:
+                case SQLITE_ROW: {
+                    int ncols = sqlite3_column_count(stmt);
+
+                    for (int lpc = 0; lpc < ncols; lpc++) {
+                        const char *name = sqlite3_column_name(stmt, lpc);
+                        const char *value = (const char *)
+                                sqlite3_column_text(stmt, lpc);
+
+                        lvars[name] = value;
+                    }
                     break;
+                }
 
                 default: {
                     const char *errmsg;
@@ -662,7 +728,24 @@ void sql_execute_script(sqlite3 *db,
             }
         }
 
-        script = tail;
+        sqlite3_reset(stmt);
+    }
+}
+
+void sql_execute_script(sqlite3 *db,
+                        const char *src_name,
+                        const char *script,
+                        std::vector<std::string> &errors)
+{
+    vector<sqlite3_stmt *> stmts;
+
+    sql_compile_script(db, src_name, script, stmts, errors);
+    if (errors.empty()) {
+        sql_execute_script(db, stmts, errors);
+    }
+
+    for (sqlite3_stmt *stmt : stmts) {
+        sqlite3_finalize(stmt);
     }
 }
 
@@ -727,3 +810,113 @@ int sqlite_authorizer(void *pUserData, int action_code, const char *detail1,
     }
     return SQLITE_OK;
 }
+
+static string sql_keyword_re(void)
+{
+    string retval = "(?:";
+
+    for (int lpc = 0; sql_keywords[lpc]; lpc++) {
+        if (lpc > 0) {
+            retval.append("|");
+        }
+        retval.append("\\b");
+        retval.append(sql_keywords[lpc]);
+        retval.append("\\b");
+    }
+    retval += ")";
+
+    return retval;
+}
+
+string_attr_type SQL_KEYWORD_ATTR("sql_keyword");
+string_attr_type SQL_IDENTIFIER_ATTR("sql_ident");
+string_attr_type SQL_FUNCTION_ATTR("sql_func");
+string_attr_type SQL_STRING_ATTR("sql_string");
+string_attr_type SQL_OPERATOR_ATTR("sql_oper");
+string_attr_type SQL_PAREN_ATTR("sql_paren");
+string_attr_type SQL_GARBAGE_ATTR("sql_garbage");
+
+void annotate_sql_statement(attr_line_t &al)
+{
+    static string keyword_re_str =
+        R"(\A)" + sql_keyword_re() + R"(|\.schema|\.msgformats)";
+
+    static struct {
+        pcrepp re;
+        string_attr_type_t type;
+    } PATTERNS[] = {
+        { {keyword_re_str.c_str(), PCRE_CASELESS}, &SQL_KEYWORD_ATTR },
+        { {R"(\A'[^']*('(?:'[^']*')*|$))"}, &SQL_STRING_ATTR },
+        { {R"(\A(\$?\b[a-z_]\w*)|\"([^\"]+)\"|\[([^\]]+)])", PCRE_CASELESS}, &SQL_IDENTIFIER_ATTR },
+        { {R"(\A(\*|<|>|=|!|\-|\+|\|\|))"}, &SQL_OPERATOR_ATTR },
+        { {R"(\A\(|\))"}, &SQL_PAREN_ATTR },
+        { {R"(\A.)"}, &SQL_GARBAGE_ATTR },
+    };
+
+    static pcrepp ws_pattern(R"(\A\s+)");
+
+    pcre_context_static<30> pc;
+    pcre_input pi(al.get_string());
+    string &line = al.get_string();
+    string_attrs_t &sa = al.get_attrs();
+
+    while (pi.pi_next_offset < line.length()) {
+        if (ws_pattern.match(pc, pi, PCRE_ANCHORED)) {
+            continue;
+        }
+        for (auto &pat : PATTERNS) {
+            if (pat.re.match(pc, pi, PCRE_ANCHORED)) {
+                pcre_context::capture_t *cap = pc.all();
+                struct line_range lr(cap->c_begin, cap->c_end);
+
+                sa.emplace_back(lr, pat.type);
+                break;
+            }
+        }
+    }
+
+    string_attrs_t::const_iterator iter;
+    int start = 0;
+
+    while ((iter = find_string_attr(sa, &SQL_IDENTIFIER_ATTR, start)) != sa.end()) {
+        string_attrs_t::const_iterator piter;
+        bool found_open = false;
+        ssize_t lpc;
+
+        for (lpc = iter->sa_range.lr_end; lpc < line.length(); lpc++) {
+            if (line[lpc] == '(') {
+                found_open = true;
+                break;
+            } else if (!isspace(line[lpc])) {
+                break;
+            }
+        }
+
+        if (found_open) {
+            ssize_t pstart = lpc + 1;
+            int depth = 1;
+
+            while (depth > 0 &&
+                   (piter = find_string_attr(sa, &SQL_PAREN_ATTR, pstart)) != sa.end()) {
+                if (line[piter->sa_range.lr_start] == '(') {
+                    depth += 1;
+                } else {
+                    depth -= 1;
+                }
+                pstart = piter->sa_range.lr_end;
+            }
+
+            line_range func_range{iter->sa_range.lr_start};
+            if (piter == sa.end()) {
+                func_range.lr_end = line.length();
+            } else {
+                func_range.lr_end = piter->sa_range.lr_end;
+            }
+            sa.emplace_back(func_range, &SQL_FUNCTION_ATTR);
+        }
+
+        start = iter->sa_range.lr_end;
+    }
+
+    remove_string_attr(sa, &SQL_PAREN_ATTR);
+}
