@@ -38,6 +38,7 @@
 #include "ansi_scrubber.hh"
 #include "log_format.hh"
 #include "textview_curses.hh"
+#include "view_curses.hh"
 
 using namespace std;
 
@@ -59,16 +60,20 @@ bookmark_type_t textview_curses::BM_SEARCH("search");
 
 string_attr_type textview_curses::SA_ORIGINAL_LINE;
 string_attr_type textview_curses::SA_BODY;
+string_attr_type textview_curses::SA_HIDDEN;
+string_attr_type textview_curses::SA_FORMAT;
 
 textview_curses::textview_curses()
     : tc_sub_source(NULL),
       tc_delegate(NULL),
       tc_searching(false),
-      tc_follow_search(false),
       tc_selection_start(-1),
       tc_selection_last(-1),
-      tc_selection_cleared(false)
+      tc_selection_cleared(false),
+      tc_hide_fields(true)
 {
+    this->tc_follow_deadline.tv_sec = 0;
+    this->tc_follow_deadline.tv_usec = 0;
     this->set_data_source(this);
 }
 
@@ -112,148 +117,13 @@ void textview_curses::grep_match(grep_proc &gp,
     }
 }
 
-void textview_curses::listview_value_for_row(const listview_curses &lv,
-                                             vis_line_t row,
-                                             attr_line_t &value_out)
+void textview_curses::listview_value_for_rows(const listview_curses &lv,
+                                              vis_line_t row,
+                                              vector<attr_line_t> &rows_out)
 {
-    bookmark_vector<vis_line_t> &user_marks = this->tc_bookmarks[&BM_USER];
-    bookmark_vector<vis_line_t> &part_marks = this->tc_bookmarks[&BM_PARTITION];
-    string_attrs_t &             sa         = value_out.get_attrs();
-    string &                     str        = value_out.get_string();
-    highlight_map_t::iterator    iter;
-    string::iterator             str_iter;
-
-    this->tc_sub_source->text_value_for_line(*this, row, str);
-    this->tc_sub_source->text_attrs_for_line(*this, row, sa);
-
-    scrub_ansi_string(str, sa);
-
-    struct line_range body;
-
-    body = find_string_attr_range(sa, &SA_BODY);
-    if (body.lr_start == -1) {
-        body.lr_start = 0;
-        body.lr_end = str.size();
-    }
-
-    for (iter = this->tc_highlights.begin();
-         iter != this->tc_highlights.end();
-         iter++) {
-        // XXX testing for '$search' here sucks
-        bool internal_hl = iter->first[0] == '$' && iter->first != "$search";
-        int off, hcount = 0;
-        size_t re_end;
-
-        if (body.lr_end > 8192)
-            re_end = 8192;
-        else
-            re_end = body.lr_end;
-
-        for (off = internal_hl ? body.lr_start : 0; off < (int)str.size(); ) {
-            int rc, matches[60];
-            rc = pcre_exec(iter->second.h_code,
-                           iter->second.h_code_extra,
-                           str.c_str(),
-                           re_end,
-                           off,
-                           0,
-                           matches,
-                           60);
-            if (rc > 0) {
-                struct line_range lr;
-
-                if (rc == 2) {
-                    lr.lr_start = matches[2];
-                    lr.lr_end   = matches[3];
-                }
-                else {
-                    lr.lr_start = matches[0];
-                    lr.lr_end   = matches[1];
-                }
-
-                if (lr.lr_end > lr.lr_start) {
-                    sa.push_back(string_attr(
-                        lr, &view_curses::VC_STYLE, iter->second.get_attrs(hcount)));
-                    hcount++;
-
-                    off = matches[1];
-                }
-                else {
-                    off += 1;
-                }
-            }
-            else {
-                off = str.size();
-            }
-        }
-    }
-
-#if 0
-    typedef std::map<std::string, view_colors::role_t> key_map_t;
-    static key_map_t key_roles;
-
-    data_scanner ds(str);
-    data_parser  dp(&ds);
-
-    dp.parse();
-
-    for (list<data_parser::element>::iterator iter = dp.dp_stack.begin();
-         iter != dp.dp_stack.end();
-         ++iter) {
-        view_colors &vc = view_colors::singleton();
-
-        if (iter->e_token == DNT_PAIR) {
-            list<data_parser::element>::iterator pair_iter;
-            key_map_t::iterator km_iter;
-            data_token_t        value_token;
-            struct line_range   lr;
-            string key;
-
-            value_token =
-                iter->e_sub_elements->back().e_sub_elements->front().e_token;
-            if (value_token == DT_STRING) {
-                continue;
-            }
-
-            lr.lr_start = iter->e_capture.c_begin;
-            lr.lr_end   = iter->e_capture.c_end;
-
-            key = ds.get_input().get_substr(
-                &iter->e_sub_elements->front().e_capture);
-            if ((km_iter = key_roles.find(key)) == key_roles.end()) {
-                key_roles[key] = vc.next_highlight();
-            }
-            /* fprintf(stderr, "key = %s\n", key.c_str()); */
-            sa[lr].insert(make_string_attr("style",
-                                           vc.attrs_for_role(key_roles[key])));
-
-            pair_iter = iter->e_sub_elements->begin();
-            ++pair_iter;
-
-            lr.lr_start = pair_iter->e_capture.c_begin;
-            lr.lr_end   = pair_iter->e_capture.c_end;
-            sa[lr].insert(make_string_attr("style",
-                                           COLOR_PAIR(view_colors::VC_WHITE) |
-                                           A_BOLD));
-        }
-    }
-#endif
-
-    if (binary_search(user_marks.begin(), user_marks.end(), row)) {
-        struct line_range        lr(0);
-        string_attrs_t::iterator iter;
-
-        for (iter = sa.begin(); iter != sa.end(); iter++) {
-            if (iter->sa_type == &view_curses::VC_STYLE) {
-                iter->sa_value.sav_int ^= A_REVERSE;
-            }
-        }
-
-        sa.push_back(string_attr(lr, &view_curses::VC_STYLE, A_REVERSE));
-    }
-    else if (binary_search(part_marks.begin(), part_marks.end(), row + 1)) {
-        sa.push_back(string_attr(
-            line_range(0), &view_curses::VC_STYLE, A_UNDERLINE));
+    for (auto &al : rows_out) {
+        this->textview_value_for_row(row, al);
+        ++row;
     }
 }
 
@@ -338,3 +208,198 @@ bool textview_curses::handle_mouse(mouse_event &me)
 
     return true;
 }
+
+void textview_curses::textview_value_for_row(vis_line_t row,
+                                             attr_line_t &value_out)
+{
+    view_colors &vc = view_colors::singleton();
+    bookmark_vector<vis_line_t> &user_marks = this->tc_bookmarks[&BM_USER];
+    bookmark_vector<vis_line_t> &part_marks = this->tc_bookmarks[&BM_PARTITION];
+    string_attrs_t &             sa         = value_out.get_attrs();
+    string &                     str        = value_out.get_string();
+    highlight_map_t::iterator    iter;
+    text_format_t source_format = this->tc_sub_source->get_text_format();
+    intern_string_t format_name;
+
+    this->tc_sub_source->text_value_for_line(*this, row, str);
+    this->tc_sub_source->text_attrs_for_line(*this, row, sa);
+
+    scrub_ansi_string(str, sa);
+
+    struct line_range body, orig_line;
+
+    body = find_string_attr_range(sa, &SA_BODY);
+    if (body.lr_start == -1) {
+        body.lr_start = 0;
+        body.lr_end = str.size();
+    }
+
+    orig_line = find_string_attr_range(sa, &SA_ORIGINAL_LINE);
+    if (!orig_line.is_valid()) {
+        orig_line.lr_start = 0;
+        orig_line.lr_end = str.size();
+    }
+
+    auto sa_iter = find_string_attr(sa, &SA_FORMAT);
+    if (sa_iter != sa.end()) {
+        format_name = sa_iter->to_string();
+    }
+
+    for (iter = this->tc_highlights.begin();
+         iter != this->tc_highlights.end();
+         iter++) {
+        // XXX testing for '$search' here sucks
+        bool internal_hl = iter->first[0] == '$'
+                           && iter->first != "$search"
+                           && iter->first != "$preview";
+        int off;
+        size_t re_end;
+
+        if (body.lr_end > 8192)
+            re_end = 8192;
+        else
+            re_end = body.lr_end;
+
+        if (iter->second.h_text_format != TF_UNKNOWN &&
+            source_format != iter->second.h_text_format) {
+            continue;
+        }
+
+        if (!iter->second.h_format_name.empty() &&
+            iter->second.h_format_name != format_name) {
+            continue;
+        }
+
+        for (off = internal_hl ? body.lr_start : 0; off < (int)str.size(); ) {
+            int rc, matches[60];
+            rc = pcre_exec(iter->second.h_code,
+                           iter->second.h_code_extra,
+                           str.c_str(),
+                           re_end,
+                           off,
+                           0,
+                           matches,
+                           60);
+            if (rc > 0) {
+                struct line_range lr;
+
+                if (rc == 2) {
+                    lr.lr_start = matches[2];
+                    lr.lr_end   = matches[3];
+                }
+                else {
+                    lr.lr_start = matches[0];
+                    lr.lr_end   = matches[1];
+                }
+
+                if (lr.lr_end > lr.lr_start) {
+                    sa.push_back(string_attr(
+                        lr, &view_curses::VC_STYLE, iter->second.get_attrs()));
+
+                    off = matches[1];
+                }
+                else {
+                    off += 1;
+                }
+            }
+            else {
+                off = str.size();
+            }
+        }
+    }
+
+    if (this->tc_hide_fields) {
+        for (auto &sattr : sa) {
+            if (sattr.sa_type == &SA_HIDDEN &&
+                sattr.sa_range.length() > 3) {
+                struct line_range &lr = sattr.sa_range;
+
+                str.replace(lr.lr_start, lr.length(), "...");
+                shift_string_attrs(sa, lr.lr_start + 1, -(lr.length() - 3));
+                sattr.sa_type = &VC_GRAPHIC;
+                sattr.sa_value.sav_int =
+                    vc.ansi_color_pair(COLOR_YELLOW, COLOR_BLACK) |
+                    ACS_BULLET | A_UNDERLINE;
+                lr.lr_end = lr.lr_start + 3;
+
+                for_each(sa.begin(), sa.end(), [&] (string_attr &attr) {
+                    if (attr.sa_type == &VC_STYLE &&
+                        attr.sa_range.lr_start == lr.lr_start) {
+                        attr.sa_type = NULL;
+                    }
+                });
+            }
+        }
+    }
+
+#if 0
+    typedef std::map<std::string, view_colors::role_t> key_map_t;
+    static key_map_t key_roles;
+
+    data_scanner ds(str);
+    data_parser  dp(&ds);
+
+    dp.parse();
+
+    for (list<data_parser::element>::iterator iter = dp.dp_stack.begin();
+         iter != dp.dp_stack.end();
+         ++iter) {
+        view_colors &vc = view_colors::singleton();
+
+        if (iter->e_token == DNT_PAIR) {
+            list<data_parser::element>::iterator pair_iter;
+            key_map_t::iterator km_iter;
+            data_token_t        value_token;
+            struct line_range   lr;
+            string key;
+
+            value_token =
+                iter->e_sub_elements->back().e_sub_elements->front().e_token;
+            if (value_token == DT_STRING) {
+                continue;
+            }
+
+            lr.lr_start = iter->e_capture.c_begin;
+            lr.lr_end   = iter->e_capture.c_end;
+
+            key = ds.get_input().get_substr(
+                &iter->e_sub_elements->front().e_capture);
+            if ((km_iter = key_roles.find(key)) == key_roles.end()) {
+                key_roles[key] = vc.next_highlight();
+            }
+            /* fprintf(stderr, "key = %s\n", key.c_str()); */
+            sa[lr].insert(make_string_attr("style",
+                                           vc.attrs_for_role(key_roles[key])));
+
+            pair_iter = iter->e_sub_elements->begin();
+            ++pair_iter;
+
+            lr.lr_start = pair_iter->e_capture.c_begin;
+            lr.lr_end   = pair_iter->e_capture.c_end;
+            sa[lr].insert(make_string_attr("style",
+                                           COLOR_PAIR(view_colors::VC_WHITE) |
+                                           A_BOLD));
+        }
+    }
+#endif
+
+    if (binary_search(user_marks.begin(), user_marks.end(), row)) {
+        string_attrs_t::iterator iter;
+
+        for (iter = sa.begin(); iter != sa.end(); iter++) {
+            if (iter->sa_range.lr_start < orig_line.lr_start) {
+                continue;
+            }
+
+            if (iter->sa_type == &view_curses::VC_STYLE) {
+                iter->sa_value.sav_int ^= A_REVERSE;
+            }
+        }
+
+        sa.emplace_back(orig_line, &view_curses::VC_STYLE, A_REVERSE);
+    }
+    else if (binary_search(part_marks.begin(), part_marks.end(), row + 1)) {
+        sa.push_back(string_attr(
+            line_range(0), &view_curses::VC_STYLE, A_UNDERLINE));
+    }
+}
