@@ -7,7 +7,8 @@
 
 #include "lnav.hh"
 #include "auto_mem.hh"
-#include "sqlite-extension-func.h"
+#include "sqlite-extension-func.hh"
+#include "regexp_vtab.hh"
 
 struct callback_state {
     int cs_row;
@@ -32,6 +33,11 @@ static int sql_callback(void *ptr,
     return 0;
 }
 
+std::string execute_any(exec_context &ec, const std::string &cmdline_with_mode)
+{
+    return "";
+}
+
 int main(int argc, char *argv[])
 {
     int retval = EXIT_SUCCESS;
@@ -60,6 +66,8 @@ int main(int argc, char *argv[])
             register_collation_functions(db.in());
         }
 
+        register_regexp_vtab(db.in());
+
         if (sqlite3_exec(db.in(),
             argv[1],
             sql_callback,
