@@ -47,6 +47,11 @@ time_t time(time_t *arg)
     return current_time;
 }
 
+string execute_any(exec_context &ec, const string &cmdline_with_mode)
+{
+    return "";
+}
+
 int main(int argc, char *argv[])
 {
     int retval = EXIT_SUCCESS;
@@ -74,7 +79,7 @@ int main(int argc, char *argv[])
         lnav_config.lc_ui_clock_format = "abc";
         tss.update_time();
         val = sf.get_value();
-        assert(val.get_string() == "abc");
+        assert(val.get_string() == " abc");
     }
 
 	return retval;
